@@ -1,4 +1,4 @@
-use hir::HirDisplay;
+use hir::{HasSource, HirDisplay, Module, TypeInfo};
 use ide_db::{base_db::FileId, helpers::SnippetCap};
 use rustc_hash::{FxHashMap, FxHashSet};
 use stdx::to_lower_snake_case;
@@ -6,14 +6,14 @@ use syntax::{
     ast::{
         self,
         edit::{AstNodeEdit, IndentLevel},
-        make, ArgListOwner, AstNode, ModuleItemOwner,
+        make, ArgListOwner, AstNode, CallExpr, ModuleItemOwner,
     },
-    SyntaxKind, SyntaxNode, TextSize,
+    SyntaxKind, SyntaxNode, TextRange, TextSize,
 };
 
 use crate::{
     utils::useless_type_special_case,
-    utils::{render_snippet, Cursor},
+    utils::{find_struct_impl, render_snippet, Cursor},
     AssistContext, AssistId, AssistKind, Assists,
 };
 
@@ -43,6 +43,31 @@ use crate::{
 //
 // ```
 pub(crate) fn generate_function(acc: &mut Assists, ctx: &AssistContext) -> Option<()> {
+    gen_fn(acc, ctx).or_else(|| gen_method(acc, ctx))
+}
+
+enum FuncExpr {
+    Func(ast::CallExpr),
+    Method(ast::MethodCallExpr),
+}
+
+impl FuncExpr {
+    fn arg_list(&self) -> Option<ast::ArgList> {
+        match self {
+            FuncExpr::Func(fn_call) => fn_call.arg_list(),
+            FuncExpr::Method(m_call) => m_call.arg_list(),
+        }
+    }
+
+    fn syntax(&self) -> &SyntaxNode {
+        match self {
+            FuncExpr::Func(fn_call) => fn_call.syntax(),
+            FuncExpr::Method(m_call) => m_call.syntax(),
+        }
+    }
+}
+
+fn gen_fn(acc: &mut Assists, ctx: &AssistContext) -> Option<()> {
     let path_expr: ast::PathExpr = ctx.find_node_at_offset()?;
     let call = path_expr.syntax().parent().and_then(ast::CallExpr::cast)?;
 
@@ -60,43 +85,105 @@ pub(crate) fn generate_function(acc: &mut Assists, ctx: &AssistContext) -> Optio
         None => None,
     };
 
-    let function_builder = FunctionBuilder::from_call(ctx, &call, &path, target_module)?;
+    let (target, file, insert_offset) = get_fn_target(ctx, &target_module, call.clone())?;
+    let function_builder = FunctionBuilder::from_call(ctx, &call, &path, target_module, target)?;
     let target = call.syntax().text_range();
+    let label = format!("Generate {} function", function_builder.fn_name.clone());
+    add_func_to_accumulator(acc, ctx, target, function_builder, insert_offset, file, None, label)
+}
 
-    acc.add(
-        AssistId("generate_function", AssistKind::Generate),
-        format!("Generate `{}` function", function_builder.fn_name),
-        target,
-        |builder| {
-            let function_template = function_builder.render();
-            builder.edit_file(function_template.file);
-            let new_fn = function_template.to_string(ctx.config.snippet_cap);
-            match ctx.config.snippet_cap {
-                Some(cap) => builder.insert_snippet(cap, function_template.insert_offset, new_fn),
-                None => builder.insert(function_template.insert_offset, new_fn),
-            }
-        },
+fn gen_method(acc: &mut Assists, ctx: &AssistContext) -> Option<()> {
+    let call: ast::MethodCallExpr = ctx.find_node_at_offset()?;
+    let fn_name = call.name_ref()?;
+    let adt = ctx.sema.type_of_expr(&call.receiver()?)?.original().strip_references().as_adt()?;
+
+    let current_module = current_module(call.syntax(), ctx)?;
+    let target_module = adt.module(ctx.sema.db);
+
+    if current_module.krate() != target_module.krate() {
+        return None;
+    }
+
+    let range = adt.source(ctx.sema.db)?.syntax().original_file_range(ctx.sema.db);
+    let file = ctx.sema.parse(range.file_id);
+    let adt_source =
+        ctx.sema.find_node_at_offset_with_macros(file.syntax(), range.range.start())?;
+    let impl_ = find_struct_impl(ctx, &adt_source, fn_name.text().as_str())?;
+
+    let (target, insert_offset) = get_method_target(ctx, &target_module, &impl_)?;
+    let function_builder =
+        FunctionBuilder::from_method_call(ctx, &call, &fn_name, target_module, target)?;
+    let text_range = call.syntax().text_range();
+    let adt_name = if impl_.is_none() { Some(adt.name(ctx.sema.db)) } else { None };
+    let label = format!("Generate {} method", function_builder.fn_name.clone());
+    add_func_to_accumulator(
+        acc,
+        ctx,
+        text_range,
+        function_builder,
+        insert_offset,
+        range.file_id,
+        adt_name,
+        label,
     )
 }
 
-struct FunctionTemplate {
+fn add_func_to_accumulator(
+    acc: &mut Assists,
+    ctx: &AssistContext,
+    text_range: TextRange,
+    function_builder: FunctionBuilder,
     insert_offset: TextSize,
+    file: FileId,
+    adt_name: Option<hir::Name>,
+    label: String,
+) -> Option<()> {
+    acc.add(AssistId("generate_function", AssistKind::Generate), label, text_range, |builder| {
+        let function_template = function_builder.render();
+        let mut func = function_template.to_string(ctx.config.snippet_cap);
+        if let Some(name) = adt_name {
+            func = format!("\nimpl {} {{\n{}\n}}", name, func);
+        }
+        builder.edit_file(file);
+        match ctx.config.snippet_cap {
+            Some(cap) => builder.insert_snippet(cap, insert_offset, func),
+            None => builder.insert(insert_offset, func),
+        }
+    })
+}
+
+fn current_module(current_node: &SyntaxNode, ctx: &AssistContext) -> Option<Module> {
+    ctx.sema.scope(current_node).module()
+}
+
+struct FunctionTemplate {
     leading_ws: String,
     fn_def: ast::Fn,
-    ret_type: ast::RetType,
-    should_render_snippet: bool,
+    ret_type: Option<ast::RetType>,
+    should_focus_return_type: bool,
     trailing_ws: String,
-    file: FileId,
+    tail_expr: ast::Expr,
 }
 
 impl FunctionTemplate {
     fn to_string(&self, cap: Option<SnippetCap>) -> String {
-        let f = match (cap, self.should_render_snippet) {
-            (Some(cap), true) => {
-                render_snippet(cap, self.fn_def.syntax(), Cursor::Replace(self.ret_type.syntax()))
+        let f = match cap {
+            Some(cap) => {
+                let cursor = if self.should_focus_return_type {
+                    // Focus the return type if there is one
+                    if let Some(ref ret_type) = self.ret_type {
+                        ret_type.syntax()
+                    } else {
+                        self.tail_expr.syntax()
+                    }
+                } else {
+                    self.tail_expr.syntax()
+                };
+                render_snippet(cap, self.fn_def.syntax(), Cursor::Replace(cursor))
             }
-            _ => self.fn_def.to_string(),
+            None => self.fn_def.to_string(),
         };
+
         format!("{}{}{}", self.leading_ws, f, self.trailing_ws)
     }
 }
@@ -106,9 +193,8 @@ struct FunctionBuilder {
     fn_name: ast::Name,
     type_params: Option<ast::GenericParamList>,
     params: ast::ParamList,
-    ret_type: ast::RetType,
-    should_render_snippet: bool,
-    file: FileId,
+    ret_type: Option<ast::RetType>,
+    should_focus_return_type: bool,
     needs_pub: bool,
     is_async: bool,
 }
@@ -121,51 +207,18 @@ impl FunctionBuilder {
         call: &ast::CallExpr,
         path: &ast::Path,
         target_module: Option<hir::Module>,
+        target: GeneratedFunctionTarget,
     ) -> Option<Self> {
-        let mut file = ctx.frange.file_id;
-        let target = match &target_module {
-            Some(target_module) => {
-                let module_source = target_module.definition_source(ctx.db());
-                let (in_file, target) = next_space_for_fn_in_module(ctx.sema.db, &module_source)?;
-                file = in_file;
-                target
-            }
-            None => next_space_for_fn_after_call_site(call)?,
-        };
         let needs_pub = target_module.is_some();
-        let target_module = target_module.or_else(|| ctx.sema.scope(target.syntax()).module())?;
+        let target_module = target_module.or_else(|| current_module(target.syntax(), ctx))?;
         let fn_name = fn_name(path)?;
-        let (type_params, params) = fn_args(ctx, target_module, call)?;
+        let (type_params, params) = fn_args(ctx, target_module, FuncExpr::Func(call.clone()))?;
 
         let await_expr = call.syntax().parent().and_then(ast::AwaitExpr::cast);
         let is_async = await_expr.is_some();
 
-        // should_render_snippet intends to express a rough level of confidence about
-        // the correctness of the return type.
-        //
-        // If we are able to infer some return type, and that return type is not unit, we
-        // don't want to render the snippet. The assumption here is in this situation the
-        // return type is just as likely to be correct as any other part of the generated
-        // function.
-        //
-        // In the case where the return type is inferred as unit it is likely that the
-        // user does in fact intend for this generated function to return some non unit
-        // type, but that the current state of their code doesn't allow that return type
-        // to be accurately inferred.
-        let (ret_ty, should_render_snippet) = {
-            match ctx.sema.type_of_expr(&ast::Expr::CallExpr(call.clone())) {
-                Some(ty) if ty.is_unknown() || ty.is_unit() => (make::ty_unit(), true),
-                Some(ty) => {
-                    let rendered = ty.display_source_code(ctx.db(), target_module.into());
-                    match rendered {
-                        Ok(rendered) => (make::ty(&rendered), false),
-                        Err(_) => (make::ty_unit(), true),
-                    }
-                }
-                None => (make::ty_unit(), true),
-            }
-        };
-        let ret_type = make::ret_type(ret_ty);
+        let (ret_type, should_focus_return_type) =
+            make_return_type(ctx, &ast::Expr::CallExpr(call.clone()), target_module);
 
         Some(Self {
             target,
@@ -173,8 +226,37 @@ impl FunctionBuilder {
             type_params,
             params,
             ret_type,
-            should_render_snippet,
-            file,
+            should_focus_return_type,
+            needs_pub,
+            is_async,
+        })
+    }
+
+    fn from_method_call(
+        ctx: &AssistContext,
+        call: &ast::MethodCallExpr,
+        name: &ast::NameRef,
+        target_module: Module,
+        target: GeneratedFunctionTarget,
+    ) -> Option<Self> {
+        let needs_pub =
+            !module_is_descendant(&current_module(call.syntax(), ctx)?, &target_module, ctx);
+        let fn_name = make::name(&name.text());
+        let (type_params, params) = fn_args(ctx, target_module, FuncExpr::Method(call.clone()))?;
+
+        let await_expr = call.syntax().parent().and_then(ast::AwaitExpr::cast);
+        let is_async = await_expr.is_some();
+
+        let (ret_type, should_focus_return_type) =
+            make_return_type(ctx, &ast::Expr::MethodCallExpr(call.clone()), target_module);
+
+        Some(Self {
+            target,
+            fn_name,
+            type_params,
+            params,
+            ret_type,
+            should_focus_return_type,
             needs_pub,
             is_async,
         })
@@ -190,41 +272,112 @@ impl FunctionBuilder {
             self.type_params,
             self.params,
             fn_body,
-            Some(self.ret_type),
+            self.ret_type,
             self.is_async,
         );
         let leading_ws;
         let trailing_ws;
 
-        let insert_offset = match self.target {
+        match self.target {
             GeneratedFunctionTarget::BehindItem(it) => {
                 let indent = IndentLevel::from_node(&it);
                 leading_ws = format!("\n\n{}", indent);
                 fn_def = fn_def.indent(indent);
                 trailing_ws = String::new();
-                it.text_range().end()
             }
             GeneratedFunctionTarget::InEmptyItemList(it) => {
                 let indent = IndentLevel::from_node(&it);
                 leading_ws = format!("\n{}", indent + 1);
                 fn_def = fn_def.indent(indent + 1);
                 trailing_ws = format!("\n{}", indent);
-                it.text_range().start() + TextSize::of('{')
             }
         };
 
         FunctionTemplate {
-            insert_offset,
             leading_ws,
-            ret_type: fn_def.ret_type().unwrap(),
-            should_render_snippet: self.should_render_snippet,
+            ret_type: fn_def.ret_type(),
+            // PANIC: we guarantee we always create a function body with a tail expr
+            tail_expr: fn_def.body().unwrap().tail_expr().unwrap(),
+            should_focus_return_type: self.should_focus_return_type,
             fn_def,
             trailing_ws,
-            file: self.file,
         }
     }
 }
 
+/// Makes an optional return type along with whether the return type should be focused by the cursor.
+/// If we cannot infer what the return type should be, we create unit as a placeholder.
+///
+/// The rule for whether we focus a return type or not (and thus focus the function body),
+/// is rather simple:
+/// * If we could *not* infer what the return type should be, focus it (so the user can fill-in
+/// the correct return type).
+/// * If we could infer the return type, don't focus it (and thus focus the function body) so the
+/// user can change the `todo!` function body.
+fn make_return_type(
+    ctx: &AssistContext,
+    call: &ast::Expr,
+    target_module: Module,
+) -> (Option<ast::RetType>, bool) {
+    let (ret_ty, should_focus_return_type) = {
+        match ctx.sema.type_of_expr(call).map(TypeInfo::original) {
+            Some(ty) if ty.is_unknown() => (Some(make::ty_unit()), true),
+            None => (Some(make::ty_unit()), true),
+            Some(ty) if ty.is_unit() => (None, false),
+            Some(ty) => {
+                let rendered = ty.display_source_code(ctx.db(), target_module.into());
+                match rendered {
+                    Ok(rendered) => (Some(make::ty(&rendered)), false),
+                    Err(_) => (Some(make::ty_unit()), true),
+                }
+            }
+        }
+    };
+    let ret_type = ret_ty.map(|rt| make::ret_type(rt));
+    (ret_type, should_focus_return_type)
+}
+
+fn get_fn_target(
+    ctx: &AssistContext,
+    target_module: &Option<Module>,
+    call: CallExpr,
+) -> Option<(GeneratedFunctionTarget, FileId, TextSize)> {
+    let mut file = ctx.frange.file_id;
+    let target = match target_module {
+        Some(target_module) => {
+            let module_source = target_module.definition_source(ctx.db());
+            let (in_file, target) = next_space_for_fn_in_module(ctx.sema.db, &module_source)?;
+            file = in_file;
+            target
+        }
+        None => next_space_for_fn_after_call_site(FuncExpr::Func(call.clone()))?,
+    };
+    Some((target.clone(), file, get_insert_offset(&target)))
+}
+
+fn get_method_target(
+    ctx: &AssistContext,
+    target_module: &Module,
+    impl_: &Option<ast::Impl>,
+) -> Option<(GeneratedFunctionTarget, TextSize)> {
+    let target = match impl_ {
+        Some(impl_) => next_space_for_fn_in_impl(&impl_)?,
+        None => {
+            next_space_for_fn_in_module(ctx.sema.db, &target_module.definition_source(ctx.sema.db))?
+                .1
+        }
+    };
+    Some((target.clone(), get_insert_offset(&target)))
+}
+
+fn get_insert_offset(target: &GeneratedFunctionTarget) -> TextSize {
+    match &target {
+        GeneratedFunctionTarget::BehindItem(it) => it.text_range().end(),
+        GeneratedFunctionTarget::InEmptyItemList(it) => it.text_range().start() + TextSize::of('{'),
+    }
+}
+
+#[derive(Clone)]
 enum GeneratedFunctionTarget {
     BehindItem(SyntaxNode),
     InEmptyItemList(SyntaxNode),
@@ -248,7 +401,7 @@ fn fn_name(call: &ast::Path) -> Option<ast::Name> {
 fn fn_args(
     ctx: &AssistContext,
     target_module: hir::Module,
-    call: &ast::CallExpr,
+    call: FuncExpr,
 ) -> Option<(Option<ast::GenericParamList>, ast::ParamList)> {
     let mut arg_names = Vec::new();
     let mut arg_types = Vec::new();
@@ -276,7 +429,17 @@ fn fn_args(
     let params = arg_names.into_iter().zip(arg_types).map(|(name, ty)| {
         make::param(make::ext::simple_ident_pat(make::name(&name)).into(), make::ty(&ty))
     });
-    Some((None, make::param_list(None, params)))
+
+    Some((
+        None,
+        make::param_list(
+            match call {
+                FuncExpr::Func(_) => None,
+                FuncExpr::Method(_) => Some(make::self_param()),
+            },
+            params,
+        ),
+    ))
 }
 
 /// Makes duplicate argument names unique by appending incrementing numbers.
@@ -331,7 +494,7 @@ fn fn_arg_type(
     target_module: hir::Module,
     fn_arg: &ast::Expr,
 ) -> Option<String> {
-    let ty = ctx.sema.type_of_expr(fn_arg)?;
+    let ty = ctx.sema.type_of_expr(fn_arg)?.adjusted();
     if ty.is_unknown() {
         return None;
     }
@@ -347,7 +510,7 @@ fn fn_arg_type(
 /// directly after the current block
 /// We want to write the generated function directly after
 /// fns, impls or macro calls, but inside mods
-fn next_space_for_fn_after_call_site(expr: &ast::CallExpr) -> Option<GeneratedFunctionTarget> {
+fn next_space_for_fn_after_call_site(expr: FuncExpr) -> Option<GeneratedFunctionTarget> {
     let mut ancestors = expr.syntax().ancestors().peekable();
     let mut last_ancestor: Option<SyntaxNode> = None;
     while let Some(next_ancestor) = ancestors.next() {
@@ -400,6 +563,26 @@ fn next_space_for_fn_in_module(
     Some((file, assist_item))
 }
 
+fn next_space_for_fn_in_impl(impl_: &ast::Impl) -> Option<GeneratedFunctionTarget> {
+    if let Some(last_item) = impl_.assoc_item_list().and_then(|it| it.assoc_items().last()) {
+        Some(GeneratedFunctionTarget::BehindItem(last_item.syntax().clone()))
+    } else {
+        Some(GeneratedFunctionTarget::InEmptyItemList(impl_.assoc_item_list()?.syntax().clone()))
+    }
+}
+
+fn module_is_descendant(module: &hir::Module, ans: &hir::Module, ctx: &AssistContext) -> bool {
+    if module == ans {
+        return true;
+    }
+    for c in ans.children(ctx.sema.db) {
+        if module_is_descendant(module, &c, ctx) {
+            return true;
+        }
+    }
+    false
+}
+
 #[cfg(test)]
 mod tests {
     use crate::tests::{check_assist, check_assist_not_applicable};
@@ -603,7 +786,7 @@ impl Baz {
 }
 
 fn bar(baz: Baz) -> Baz {
-    todo!()
+    ${0:todo!()}
 }
 ",
         )
@@ -623,8 +806,8 @@ fn foo() {
     bar("bar")
 }
 
-fn bar(arg: &str) ${0:-> ()} {
-    todo!()
+fn bar(arg: &str) {
+    ${0:todo!()}
 }
 "#,
         )
@@ -644,8 +827,8 @@ fn foo() {
     bar('x')
 }
 
-fn bar(arg: char) ${0:-> ()} {
-    todo!()
+fn bar(arg: char) {
+    ${0:todo!()}
 }
 "#,
         )
@@ -665,8 +848,8 @@ fn foo() {
     bar(42)
 }
 
-fn bar(arg: i32) ${0:-> ()} {
-    todo!()
+fn bar(arg: i32) {
+    ${0:todo!()}
 }
 ",
         )
@@ -686,8 +869,8 @@ fn foo() {
     bar(42 as u8)
 }
 
-fn bar(arg: u8) ${0:-> ()} {
-    todo!()
+fn bar(arg: u8) {
+    ${0:todo!()}
 }
 ",
         )
@@ -711,8 +894,8 @@ fn foo() {
     bar(x as u8)
 }
 
-fn bar(x: u8) ${0:-> ()} {
-    todo!()
+fn bar(x: u8) {
+    ${0:todo!()}
 }
 ",
         )
@@ -734,8 +917,8 @@ fn foo() {
     bar(worble)
 }
 
-fn bar(worble: ()) ${0:-> ()} {
-    todo!()
+fn bar(worble: ()) {
+    ${0:todo!()}
 }
 ",
         )
@@ -745,7 +928,8 @@ fn bar(worble: ()) ${0:-> ()} {
     fn add_function_with_impl_trait_arg() {
         check_assist(
             generate_function,
-            r"
+            r#"
+//- minicore: sized
 trait Foo {}
 fn foo() -> impl Foo {
     todo!()
@@ -753,8 +937,8 @@ fn foo() -> impl Foo {
 fn baz() {
     $0bar(foo())
 }
-",
-            r"
+"#,
+            r#"
 trait Foo {}
 fn foo() -> impl Foo {
     todo!()
@@ -763,10 +947,10 @@ fn baz() {
     bar(foo())
 }
 
-fn bar(foo: impl Foo) ${0:-> ()} {
-    todo!()
+fn bar(foo: impl Foo) {
+    ${0:todo!()}
 }
-",
+"#,
         )
     }
 
@@ -790,8 +974,8 @@ fn foo() {
     bar(&baz())
 }
 
-fn bar(baz: &Baz) ${0:-> ()} {
-    todo!()
+fn bar(baz: &Baz) {
+    ${0:todo!()}
 }
 ",
         )
@@ -819,8 +1003,8 @@ fn foo() {
     bar(Baz::baz())
 }
 
-fn bar(baz: Baz::Bof) ${0:-> ()} {
-    todo!()
+fn bar(baz: Baz::Bof) {
+    ${0:todo!()}
 }
 ",
         )
@@ -841,8 +1025,8 @@ fn foo<T>(t: T) {
     bar(t)
 }
 
-fn bar(t: T) ${0:-> ()} {
-    todo!()
+fn bar(t: T) {
+    ${0:todo!()}
 }
 ",
         )
@@ -895,8 +1079,8 @@ fn foo() {
     bar(closure)
 }
 
-fn bar(closure: ()) ${0:-> ()} {
-    todo!()
+fn bar(closure: ()) {
+    ${0:todo!()}
 }
 ",
         )
@@ -916,8 +1100,8 @@ fn foo() {
     bar(baz)
 }
 
-fn bar(baz: ()) ${0:-> ()} {
-    todo!()
+fn bar(baz: ()) {
+    ${0:todo!()}
 }
 ",
         )
@@ -941,8 +1125,8 @@ fn foo() {
     bar(baz(), baz())
 }
 
-fn bar(baz_1: Baz, baz_2: Baz) ${0:-> ()} {
-    todo!()
+fn bar(baz_1: Baz, baz_2: Baz) {
+    ${0:todo!()}
 }
 ",
         )
@@ -966,8 +1150,8 @@ fn foo() {
     bar(baz(), baz(), "foo", "bar")
 }
 
-fn bar(baz_1: Baz, baz_2: Baz, arg_1: &str, arg_2: &str) ${0:-> ()} {
-    todo!()
+fn bar(baz_1: Baz, baz_2: Baz, arg_1: &str, arg_2: &str) {
+    ${0:todo!()}
 }
 "#,
         )
@@ -986,8 +1170,8 @@ fn foo() {
 ",
             r"
 mod bar {
-    pub(crate) fn my_fn() ${0:-> ()} {
-        todo!()
+    pub(crate) fn my_fn() {
+        ${0:todo!()}
     }
 }
 
@@ -1022,8 +1206,8 @@ fn bar() {
     baz(foo)
 }
 
-fn baz(foo: foo::Foo) ${0:-> ()} {
-    todo!()
+fn baz(foo: foo::Foo) {
+    ${0:todo!()}
 }
 "#,
         )
@@ -1046,8 +1230,8 @@ fn foo() {
 mod bar {
     fn something_else() {}
 
-    pub(crate) fn my_fn() ${0:-> ()} {
-        todo!()
+    pub(crate) fn my_fn() {
+        ${0:todo!()}
     }
 }
 
@@ -1074,8 +1258,8 @@ fn foo() {
             r"
 mod bar {
     mod baz {
-        pub(crate) fn my_fn() ${0:-> ()} {
-            todo!()
+        pub(crate) fn my_fn() {
+            ${0:todo!()}
         }
     }
 }
@@ -1103,8 +1287,8 @@ fn main() {
             r"
 
 
-pub(crate) fn bar() ${0:-> ()} {
-    todo!()
+pub(crate) fn bar() {
+    ${0:todo!()}
 }",
         )
     }
@@ -1124,7 +1308,7 @@ fn main() {
 }
 
 fn foo() -> u32 {
-    todo!()
+    ${0:todo!()}
 }
 ",
         )
@@ -1163,8 +1347,7 @@ fn bar(baz: ()) {}
 
     #[test]
     fn create_method_with_no_args() {
-        // FIXME: This is wrong, this should just work.
-        check_assist_not_applicable(
+        check_assist(
             generate_function,
             r#"
 struct Foo;
@@ -1173,7 +1356,19 @@ impl Foo {
         self.bar()$0;
     }
 }
-        "#,
+"#,
+            r#"
+struct Foo;
+impl Foo {
+    fn foo(&self) {
+        self.bar();
+    }
+
+    fn bar(&self) ${0:-> ()} {
+        todo!()
+    }
+}
+"#,
         )
     }
 
@@ -1194,6 +1389,131 @@ fn foo() {
 async fn bar(arg: i32) ${0:-> ()} {
     todo!()
 }
+",
+        )
+    }
+
+    #[test]
+    fn create_method() {
+        check_assist(
+            generate_function,
+            r"
+struct S;
+fn foo() {S.bar$0();}
+",
+            r"
+struct S;
+fn foo() {S.bar();}
+impl S {
+
+
+fn bar(&self) ${0:-> ()} {
+    todo!()
+}
+}
+",
+        )
+    }
+
+    #[test]
+    fn create_method_within_an_impl() {
+        check_assist(
+            generate_function,
+            r"
+struct S;
+fn foo() {S.bar$0();}
+impl S {}
+
+",
+            r"
+struct S;
+fn foo() {S.bar();}
+impl S {
+    fn bar(&self) ${0:-> ()} {
+        todo!()
+    }
+}
+
+",
+        )
+    }
+
+    #[test]
+    fn create_method_from_different_module() {
+        check_assist(
+            generate_function,
+            r"
+mod s {
+    pub struct S;
+}
+fn foo() {s::S.bar$0();}
+",
+            r"
+mod s {
+    pub struct S;
+impl S {
+
+
+    pub(crate) fn bar(&self) ${0:-> ()} {
+        todo!()
+    }
+}
+}
+fn foo() {s::S.bar();}
+",
+        )
+    }
+
+    #[test]
+    fn create_method_from_descendant_module() {
+        check_assist(
+            generate_function,
+            r"
+struct S;
+mod s {
+    fn foo() {
+        super::S.bar$0();
+    }
+}
+
+",
+            r"
+struct S;
+mod s {
+    fn foo() {
+        super::S.bar();
+    }
+}
+impl S {
+
+
+fn bar(&self) ${0:-> ()} {
+    todo!()
+}
+}
+
+",
+        )
+    }
+
+    #[test]
+    fn create_method_with_cursor_anywhere_on_call_expresion() {
+        check_assist(
+            generate_function,
+            r"
+struct S;
+fn foo() {$0S.bar();}
+",
+            r"
+struct S;
+fn foo() {S.bar();}
+impl S {
+
+
+fn bar(&self) ${0:-> ()} {
+    todo!()
+}
+}
 ",
         )
     }
